package com.raycloud.tomato.openapi;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.raycloud.tomato.openapi.domain.*;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.DateUtils;
import org.apache.http.HttpResponse;
import org.apache.http.HttpStatus;
import org.apache.http.client.HttpClient;
import org.apache.http.client.config.RequestConfig;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;

import javax.xml.bind.DatatypeConverter;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.*;

/**
 * 番茄表单开发接口 数据获取实例
 * 本示例所使用的开发环境为 Java1.8
 */
public class FqOpenApiJavaDemo {

    /**
     * 番茄表单 网页版本 openAPI 域名
     */
    public static final String NET_DOMAIN = "https://fanqier.cn";

    /**
     * 番茄表单 钉钉版本 openAPI 域名
     */
    public static final String DING_DOMAIN = "https://ding.fanqier.cn";

    /**
     * 番茄 提供的 appId
     */
    public static final String APP_ID = "5a40f13f1fd001039010d2ef";

    /**
     * 番茄 提供的 AppSecret
     */
    public static final String APP_SECRET = "g8bdiTHWGe9hlli9v5I4";

    /**
     * 过期时间设置2小时
     */
    public static final long DEFAULT_EXPIRED_TIME = 2 * 60 * 60 * 1000L;

    /**
     * 获取token
     *
     * @return
     * @throws NoSuchAlgorithmException
     */
    public static String getToken() throws NoSuchAlgorithmException {
        MessageDigest crypt = MessageDigest.getInstance("SHA-1");

        Token token = new Token();
        token.setAppid(APP_ID)
                .setNow(System.currentTimeMillis())
                .setExpired(token.getNow() + DEFAULT_EXPIRED_TIME);

        String tokenJson = JSON.toJSONString(token);
        String base64 = new String(Base64.getEncoder().encode(tokenJson.getBytes()), StandardCharsets.UTF_8);

        crypt.update((tokenJson + APP_SECRET).getBytes(StandardCharsets.UTF_8));

        return base64 + '.' + DatatypeConverter.printHexBinary(crypt.digest()).toLowerCase();
    }

    /**
     * 获取表单列表
     *
     * @return
     */
    public static String listForms(Long pageNo, Long pageSize, String token) {
        String param = "";
        if (pageNo != null && pageNo >= 0 && pageSize != null && pageSize > 0) {
            param += "page=" + pageNo + "&" + "pagesize=" + pageSize;
        }
        String url = NET_DOMAIN + "/openapi/forms" + ("".equals(param) ? "" : "?" + param);
        Map<String, String> headerMap = new HashMap<>(1);
        headerMap.put("x-open-api-token", token);
        return sendGet(url, headerMap, 1500, 1500);
    }

    /**
     * 获取表单结构
     *
     * @param formId
     * @param token
     * @return
     */
    private static String getFormStructure(String formId, String token) {
        if (StringUtils.isEmpty(formId) || StringUtils.isEmpty(token)) {
            return "fail";
        }
        String url = String.format(NET_DOMAIN + "/openapi/forms/%s", formId);
        Map<String, String> headerMap = new HashMap<>(1);
        headerMap.put("x-open-api-token", token);
        return sendGet(url, headerMap, 1500, 1500);
    }

    /**
     * 获取表单回复数据列表
     *
     * @param formId
     * @param start
     * @param end
     * @param pageNo
     * @param pageSize
     * @param token
     * @return
     */
    private static String getFormReplies(String formId, Date start, Date end, Long pageNo, Long pageSize, String token) {
        if (StringUtils.isEmpty(formId) || StringUtils.isEmpty(token)) {
            return "fail";
        }
        String param = "";
        if (start != null) {
            param += "start=" + start.getTime() + "&";
        }
        if (end != null) {
            param += "end=" + end.getTime() + "&";
        }
        if (pageNo != null && pageNo >= 0 && pageSize != null && pageSize > 0) {
            param += "page=" + pageNo + "&" + "pagesize=" + pageSize;
        }
        String url = String.format(NET_DOMAIN + "/openapi/forms/%s/datas", formId) + ("".equals(param) ? "" : "?" + param);
        Map<String, String> headerMap = new HashMap<>(1);
        headerMap.put("x-open-api-token", token);
        return sendGet(url, headerMap, 1500, 1500);
    }

    /**
     * 提交表单数据
     *
     * @return
     */
    public static String postForm(String data, String token) throws UnsupportedEncodingException, NoSuchAlgorithmException {
        Map<String, String> headerMap = new HashMap<>(1);
        headerMap.put("x-open-api-token", token);
        return sendJsonPost(NET_DOMAIN + "/openapi/forms/5d50c8bf7ef6142fb4020ba3/datas", data, headerMap, 1500, 1500);
    }

    /**
     * 发送json的post请求
     *
     * @param url
     * @param headerMap
     * @throws IOException
     */
    public static String sendGet(String url, Map<String, String> headerMap, int connectionTimeOut, int sockTimeOut) {
        HttpClient httpclient = HttpClients.createDefault();

        HttpGet httpGet = new HttpGet(url);
        //添加header
        for (Map.Entry<String, String> header : headerMap.entrySet()) {
            httpGet.addHeader(header.getKey(), header.getValue());
        }

        httpGet.setConfig(RequestConfig.custom()
                .setConnectTimeout(connectionTimeOut)
                .setConnectionRequestTimeout(connectionTimeOut)
                .setSocketTimeout(sockTimeOut).build());

        HttpResponse response = null;
        try {
            response = httpclient.execute(httpGet);
            if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
                return EntityUtils.toString(response.getEntity(), StandardCharsets.UTF_8);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }


        return "fail";
    }

    /**
     * 发送json的post请求
     *
     * @param url
     * @param json
     * @param headerMap
     * @throws IOException
     */
    public static String sendJsonPost(String url, String json, Map<String, String> headerMap, int connectionTimeOut, int sockTimeOut) {
        HttpClient httpclient = HttpClients.createDefault();

        HttpPost httpPost = new HttpPost(url);
        //添加header
        for (Map.Entry<String, String> header : headerMap.entrySet()) {
            httpPost.addHeader(header.getKey(), header.getValue());
        }

        httpPost.addHeader("Content-Type", "application/json; charset=utf-8");

        httpPost.setConfig(RequestConfig.custom()
                .setConnectTimeout(connectionTimeOut)
                .setConnectionRequestTimeout(connectionTimeOut)
                .setSocketTimeout(sockTimeOut).build());

        StringEntity se = new StringEntity(json, StandardCharsets.UTF_8);
        httpPost.setEntity(se);

        HttpResponse response = null;
        try {
            response = httpclient.execute(httpPost);
            if (response.getStatusLine().getStatusCode() == HttpStatus.SC_OK) {
                return EntityUtils.toString(response.getEntity(), StandardCharsets.UTF_8);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }


        return "fail";
    }

    /**
     * 请求是否成功
     *
     * @param rspJson
     * @return
     */
    private static boolean requestIsSuccess(JSONObject rspJson) {
        if (rspJson == null) {
            return false;
        }
        String statusJsonStr = rspJson.getString("status");
        if (StringUtils.isNoneEmpty(statusJsonStr) && "success".equals(JSONObject.parseObject(statusJsonStr).getString("message"))) {
            return true;
        }
        return false;
    }

    private static void processData() throws NoSuchAlgorithmException {
        long startTimeMillis = System.currentTimeMillis();
        //生成token
        String token = getToken();

        //获取所有已发布表单、表单结构
        Map<String, Form> formListMap = null;
        Map<String, Map<String, Items>> formStructureItemMap = null;

        long pageNo = 0;
        final long pageSize = 100;
        int currSize = 0;
        do {
            JSONObject formListJson = JSONObject.parseObject(listForms(pageNo, pageSize, token));
            if (requestIsSuccess(formListJson)) {
                JSONObject formListData = JSONObject.parseObject(formListJson.getString("data"));
                int formCount = formListData.getIntValue("count");
                if (formCount <= 0) {
                    break;
                }
                if (formStructureItemMap == null) {
                    formListMap = new HashMap<>(formCount);
                    formStructureItemMap = new HashMap<>(formCount);
                }
                List<Form> formList = JSONArray.parseArray(formListData.getString("list"), Form.class);
                if (CollectionUtils.isNotEmpty(formList)) {
                    for (Form from : formList) {
                        String fromId = from.get_id();
                        if (from.getStatus() != 9) {
                            // 0为草稿——新建后编辑中，尚未发布。1为修改中——已发布的表单再次进行编辑，尚未重现发布。9为发布后的状态
                            continue;
                        }
                        // 填充表单结构
                        int tryTimes = 0;
                        do {
                            tryTimes++;
                            JSONObject formStructureJson = null;
                            try {
                                formStructureJson = JSONObject.parseObject(getFormStructure(fromId, token));

                                if (requestIsSuccess(formStructureJson)) {
                                    FormStructure formStructure = JSON.parseObject(formStructureJson.getString("data"), FormStructure.class);
                                    if (CollectionUtils.isNotEmpty(formStructure.getItems())) {
                                        Map<String, Items> itemsMap = new HashMap<>(formStructure.getItems().size());
                                        for (Items items : formStructure.getItems()) {
                                            itemsMap.put(items.get_id(), items);
                                        }
                                        formStructureItemMap.put(fromId, itemsMap);
                                    }
                                    break;
                                } else {
                                    System.out.println("获取表单结构 请求失败 : " + fromId);
                                }
                            } catch (Exception e) {
                                System.out.println(fromId + " 保存表单结构 异常");
                                e.printStackTrace();
                            }
                        } while (tryTimes < 3);
                        System.out.println("保存表单结构 " + formListMap.size() + " 条");
                        formListMap.put(fromId, from);
                    }
                    currSize = formList.size();
                    pageNo++;
                }
            }
        } while (currSize >= pageSize);

        //循环拉取回复数据
        List<Map<String, Map<String, String>>> formRepliesMapList = new ArrayList<>();
        Set<String> formRepliesIds = new HashSet<>();
        Date now = new Date();
        while (true) {

            for (Map.Entry<String, Form> entry : formListMap.entrySet()) {
                String fromId = entry.getKey();
                System.out.println("处理回复数据: " + fromId);
                pageNo = 0;
                currSize = 0;
                do {
                    // 获取最新的表单回复数据列表
                    try {
                        JSONObject formRepliesJson = JSONObject.parseObject(getFormReplies(fromId, DateUtils.addDays(now, -10), now, 0L, 100L, token));
                        if (requestIsSuccess(formRepliesJson)) {
                            JSONObject formRepliesData = JSONObject.parseObject(formRepliesJson.getString("data"));
                            long formRepliesCount = formRepliesData.getLongValue("count");
                            if (formRepliesCount <= 0L) {
                                break;
                            }
                            List<FormReplies> formRepliesList = JSONArray.parseArray(formRepliesData.getString("list"), FormReplies.class);
                            Map<String, Items> itemsMap = formStructureItemMap.get(fromId);
                            for (FormReplies formReplies : formRepliesList) {
                                String formRepliesId = formReplies.get_id();
                                if (formRepliesIds.contains(formRepliesId)) {
                                    System.out.println("回复已处理" + formRepliesId);
                                    continue;
                                }
                                Map<String, Map<String, String>> formRepliesMap = new HashMap<>();
                                Map<String, String> formRepliesContentMap = new HashMap<>(formReplies.getValues().size());
                                for (FormReplies.Values value : formReplies.getValues()) {
                                    Items item = itemsMap.get(value.getDefinition());
                                    if (item == null) {
                                        continue;
                                    }
                                    formRepliesContentMap.put(item.getLabel(), value.getValueString());
                                }
                                formRepliesIds.add(formRepliesId);
                                formRepliesMap.put(fromId, formRepliesContentMap);
                                formRepliesMapList.add(formRepliesMap);
                            }
                            currSize = formRepliesList.size();
                            pageNo++;
                        }
                    } catch (Exception e) {
                        System.out.println(fromId + " 处理回复数据 异常");
                        e.printStackTrace();
                    }
                } while (currSize >= pageSize);
            }
            System.out.println("成功处理回复数据 " + formRepliesMapList.size() + " 条");
            try {
                //10秒拉取一次数据
                Thread.sleep(10 * 1000);
            } catch (InterruptedException e) {
                System.out.println("获取数据睡眠 出错: " + e.getMessage());
            }
            if (System.currentTimeMillis() - startTimeMillis >= DEFAULT_EXPIRED_TIME) {
                //token过期重新生成
                token = getToken();
            }
        }
    }

    public static void main(String[] args) throws UnsupportedEncodingException, NoSuchAlgorithmException {
        String token = getToken();
        System.out.println(token);
        // 获取表单列表
        System.out.println("formList: " + listForms(null, null, token));

        // 获取表单结构
        String formStructure = getFormStructure("5e4649af769b5569b856bdb4", token);
        System.out.println("formStructure: " + formStructure);

        // 获取表单回复数据列表
        System.out.println("formReplies: " + getFormReplies("5e4649af769b5569b856bdb4", null, new Date(), 0L, 100L, token));

        // 提交表单数据
        String data = "{\"values\":[{\"definition\":\"5d50c9194d2fbf42240000d3\",\"type\":\"radio\",\"selectedId\":\"5d50c9384d2fbf42240000dc\",\"isOther\":false,\"selectedItem\":\"单选3\",\"remark\":\"\"},{\"definition\":\"5d50c91d4d2fbf42240000d5\",\"type\":\"checkbox\",\"selectedIds\":[\"5d50c94d4d2fbf42240000e0\"],\"selectedItems\":[\"多选2\"],\"remarks\":[\"\"]},{\"definition\":\"5d50c9554d2fbf42240000e3\",\"type\":\"dropdown\",\"selectedId\":\"5d50c9684d2fbf42240000e6\",\"selectedItem\":\"下拉2\",\"isOther\":false},{\"definition\":\"5d50c97c4d2fbf42240000e8\",\"type\":\"text\",\"value\":\"通过api提交\",\"isNumber\":false}]}";
        System.out.println("postResult: " + postForm(data, token));

        //处理回复数据
        processData();
    }
}
